open StreamHeader

val count_forced : 'a stream -> int

(* counts the number of already computed values (Lazy.Value) in a stream *)

val unit : 'a -> 'a stream
val map : ('a -> 'b) -> 'a stream -> 'b stream
val join : 'a stream stream -> 'a stream
val bind : 'a stream -> ('a -> 'b stream) -> 'b stream
val plus : 'a stream -> 'a stream -> 'a stream
val zero : unit -> 'a stream
val make : int -> 'a -> 'a stream
val get : 'a stream -> int -> 'a
val set : 'a stream -> int -> 'a -> 'a stream

val compare : 'a stream -> 'a stream -> int

val cons : 'a -> 'a stream -> 'a stream
val snoc : 'a stream -> 'a -> 'a stream
val init : int -> (int -> 'a) -> 'a stream
val from_list : 'a list -> 'a stream
val init : int -> (int -> 'a) -> 'a stream

val left_view : 'a stream -> 'a * 'a stream
val left_head : 'a stream -> 'a
val left_tail : 'a stream -> 'a stream
val right_view : 'a stream -> 'a stream * 'a
val right_head : 'a stream -> 'a
val right_tail : 'a stream -> 'a stream

val is_empty : 'a stream -> bool
val size : 'a stream -> int
val to_list : 'a stream -> 'a list
val to_list_append : 'a list -> 'a stream -> 'a list

val reverse : 'a stream -> 'a stream
val reverse_onto : 'a stream -> 'a stream -> 'a stream
val reverse_append : 'a stream -> 'a stream -> 'a stream
val append_reverse : 'a stream -> 'a stream -> 'a stream
val reverse_iter : ('a -> unit) -> 'a stream -> unit

val concat_map : ('a -> 'b stream) -> 'a stream -> 'b stream
val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a stream -> 'b
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b stream -> 'a
val iter : ('a -> 'b) -> 'a stream -> unit

val take : int -> 'a stream -> 'a stream
val drop : int -> 'a stream -> 'a stream
val split_at : int -> 'a stream -> 'a stream * 'a stream
val sub : int -> int -> 'a stream -> 'a stream

val exists : ('a -> bool) -> 'a stream -> bool
val for_all : ('a -> bool) -> 'a stream -> bool
val find : ('a -> bool) -> 'a stream -> 'a
val filter : ('a -> bool) -> 'a stream -> 'a stream
val partition : ('a -> bool) -> 'a stream -> 'a stream * 'a stream
val take_while : ('a -> bool) -> 'a stream -> 'a stream
val drop_while : ('a -> bool) -> 'a stream -> 'a stream
val split_while : ('a -> bool) -> 'a stream -> 'a stream * 'a stream

val map_with_index : (int -> 'a -> 'b) -> 'a stream -> 'b stream
val iter_with_index : (int -> 'a -> 'b) -> 'a stream -> unit
val find_with_index : (int -> 'a -> bool) -> 'a stream -> 'a
val filter_with_index : (int -> 'a -> bool) -> 'a stream -> 'a stream
val fold_right_with_index : (int -> 'a -> 'b -> 'b) -> 'b -> 'a stream -> 'b
val fold_left_with_index : (int -> 'a -> 'b -> 'a) -> 'a -> 'b stream -> 'a
val fill : 'a stream -> int -> int -> 'a -> 'a stream
val blit : 'a stream -> int -> 'a list -> int -> int -> 'a stream

val zip : 'a stream -> 'b stream -> ('a * 'b) stream
val unzip : ('a * 'b) stream -> 'a stream * 'b stream

val empty : unit -> 'a stream
val append : 'a stream -> 'a stream -> 'a stream

module StreamSeq : Signatures.FunctionalSequence
